function m = sqdist(p, q, A)
% SQDIST      Squared Euclidean or Mahalanobis distance.
% SQDIST(p,q)   returns m(i,j) = (p(:,i) - q(:,j))'*(p(:,i) - q(:,j)).
% SQDIST(p,q,A) returns m(i,j) = (p(:,i) - q(:,j))'*A*(p(:,i) - q(:,j)).

% Written by Tom Minka

p = p';
q = q';
[d, pn] = size(p);
[d, qn] = size(q);

if pn == 0 || qn == 0
  m = zeros(pn,qn);
  return
end

if nargin == 2
  
  pmag = col_sum(p .* p);
  qmag = col_sum(q .* q);
  m = repmat(qmag, pn, 1);clear qmag;
  m = m + repmat(pmag', 1, qn);clear pmag;
  m = m - 2*p'*q;
  
else

  Ap = A*p;
  Aq = A*q;
  pmag = col_sum(p .* Ap);
  qmag = col_sum(q .* Aq);
  m = repmat(qmag, pn, 1);clear qmag;
  m = m + repmat(pmag', 1, qn);clear pmag;
  m = m - 2*p'*Aq;
  
end

function s = col_sum(x) 
% COL_SUM   Sum for each column.
% A more readable alternative to sum(x,1).
s = sum(x,1);
